####################################################################################
# Copyright (C) 2018 D. Celinska-Kopczynska <dcelinska at wne dot uw dot edu dot pl#
#                                                                                  #
# This program is free software: you can redistribute it and/or modify             #
# it under the terms of the GNU General Public License as published by             #
# the Free Software Foundation, either version 3 of the License, or                #
# (at your option) any later version.                                              #
#                                                                                  #
# This program is distributed in the hope that it will be useful,                  #
# but WITHOUT ANY WARRANTY; without even the implied warranty of                   #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                    #
# GNU General Public License for more details.                                     #
#                                                                                  #
# You should have received a copy of the GNU General Public License                #
# along with this program.  If not, see <http://www.gnu.org/licenses/>.            #
####################################################################################

####################################################################################
#                   Basics of working with data frames                             #
####################################################################################

# Basics of working with R, using help

# Working directory -- used for relative paths

getwd() # returns file path to current working directory
ls() # lists objects in the current working space
setwd("file/to/dir") # new working directory               

# WARNING! Windows users have to change "\" to "/"

help.search("normal distribution") # searching for help using keywords
??Normal                           # searching a given function

7/2
exp(3)
log(3)                             # natural logarithm

# Working with a dataset

# Loading the datasets -- see EX1

# .csv files
data <- read.table("data.csv", header=TRUE, sep= ";")

# Stata files
library(foreign)
data_stata <- read.dta("data.dta")

# A typical activity is "viewing" the dataset
# Usually, it is a good idea to look at a few rows after loading the data
# to see if the process succeeded

head(data)             # 6 first rows
head(data, 20)         # 20 first rows
tail(data)             # 6 last rows
tail(data, 20)         # 20 last rows
tail(head(data,40),25) # rows from 16 to 40
tail(data$tipolau)     # 6 last values of variable tipolau

data[1:10,]    # returns first 10 rows
data[1:10,1:3] # returns first 10 rows for first 3 columns

# Notice that data frame is a "generalization" of matrix -- that is why
# refering to an element is possible in two ways:
# either with object$variable construction
# or with indexes (the first covariate is for rows, the second is for columns)

# Labeling

# You may know this activity fromn Stata, but in R it is not practicable
# In R the name of the variable is at the same time its label. Avoid whitespaces!
# names(data)[1] <- "Family number" # bruteforce, see why it is not recommended

# Setting labels for factors
# This is how to make R interpret the values as the factors
# By deafult if read.table (or similar) finds characters, treats the variable as factor
# But in many datasets the discrete variables are coded with numerals
# so R has no incentive to "know" what type they should be
# in such a case, the analyst has to change it manually

data$gender <- factor(data$gender, levels = c(1,2), labels = c("M", "F"))
data$studio <- ordered(data$studio, levels = c(6,7,8), labels = c("bachelor", "MA", "postgrad"))

# Creating easy new variables
# A typical syntax
# object$new_variable <- expression

data$per_points <- data$votoedu/data$suedu # percent of points scored
data$age <- 2004 - data$anasc              # age in 2004
data$age2 <- (data$age)**2                 # age squared

# Changes in values of the variable

# Conditional -- we put the condition in the square brackets
# We start with deleting income which is lower than 0
# We need to set those values to missing (NAs)
data$y[data$y<0] <- NA

# count how many missing data we have
# see, this construct is similar to excel's "count_if". We have to provide
# the condition as the argument
# it may return only TRUE (1) or FALSE (0)
# that is why it works -- it sums how many times the condition is met!
sum(is.na(data$y))

# removing rows containing NAs: na.omit()
data <- na.omit(data)

data$agecat <- NA
data$agecat[data$age >70] <- 1
data$agecat[data$age >55 & data$age <=70] <- 2
data$agecat[data$age >30 & data$age <=55] <- 3
data$agecat[data$age >15 & data$age <=30] <- 4

# Using ifelse -- not always effictient!
# see https://www.rdocumentation.org/packages/base/versions/3.5.1/topics/ifelse
a = c(5,7,2,9)
ifelse(a %% 2 == 0,"even","odd")

# The ordered factor
data$agecat <- ordered(data$agecat, levels <- c(1,2,3,4), labels <- c("over 70", "over 55 below 70","over 30 below 55","over 15 below 30"))

# By default R orders the factor levels alphabetically
# Which may not be appropriate, e.g., when we have the ordered factors
# This is how to change it (once we have the labeled levels)
# levels(variable) <- c("provide", "desired", "order", "of", "levels")

# Information about dataset

# The "structure" of the dataset
str(data)

# Basic statistics
summary(data)

# sorting
# sort by income (y)
newdata <- data[order(data$y),]

# sort by y (ascending) and  agecat (descending)
newdata <-data[order(data$y, -as.numeric(data$agecat)),] 

# Write to the file

write.table(data, "data_saved.csv", quote=TRUE, sep = ';', header=TRUE, row.names=FALSE)

# Saving of history and workspace -- optional

savehistory(file="hist-file.txt") # default file is .Rhistory
save.image() # saves the workspace in .RData
